(****************************************************************************)
(* BASICS.SML                                                               *)
(* Some general purpose functions and data type                             *)
(* Copyright (C) 2004 Dale Miller                                           *)
(*                                                                          *)
(* This program is free software; you can redistribute it and/or modify     *)
(* it under the terms of the GNU General Public License as published by     *)
(* the Free Software Foundation; either version 2 of the License, or        *)
(* (at your option) any later version.                                      *)
(*                                                                          *)
(* This program is distributed in the hope that it will be useful,          *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of           *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *)
(* GNU General Public License for more details.                             *)
(*                                                                          *)
(* You should have received a copy of the GNU General Public License        *)
(* along with this code; if not, write to the Free Software Foundation,     *)
(* Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA             *)
(****************************************************************************)



signature BASICS =
sig
	exception Reduce_2_List
	val reduce_list : ('a * 'b -> 'b) -> 'b -> 'a list -> 'b
	val reduce_2_list : ('a * 'b * 'c -> 'c) -> 'c -> 'a list -> 'b list -> 'c	
	val member : ''a * ''a list -> bool	
	val join_list  : ''a list * ''a list -> ''a list		
end

(* Some list manipulation functions *)

structure Basics : BASICS =
struct

exception Reduce_2_List

fun reduce_list F init [] = init
  | reduce_list F init (x::tl)=
        F(x,reduce_list F init tl)

fun reduce_2_list F init [] [] = init
  | reduce_2_list F init (x::l) (y::k)=
        F(x,y,reduce_2_list F init l k)
  | reduce_2_list _ _ _ _ = raise Reduce_2_List

fun member(x,[]) = false
  | member(x,y::l) = 
     if x=y then true else member(x,l)

fun join_list([],k) = k
  | join_list(x::l, k) =
      let val m = join_list(l,k) in
        if member(x,k) then m else x::m
      end

end      
